<?php

namespace App\Http\Controllers;

use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use App\Models\User;

use Illuminate\Routing\Controller as BaseController;

trait ApiResponseTrait
{
    public function getProductData($product)
    {
        $productImage = $product->productImage 
            ? asset('uploads/productImage/' . $product->productImage) 
            : asset('uploads/default-user.jpg');
       
        return [
            'productId' => $product->id,
            'productName' => $product->productName,
            'price' => $product->price,
            'qty' => $product->qty,
            'detail' => $product->productDetails,
            'productImage' => $productImage,
            'userId' => $product->userId,
            'businessId' => $product->businessId ?? '',
            'productCategoryId' => $product->productCategoryId,
            'isActive' => $product->isActive,
        ];
    }
}

class Controller extends BaseController
{
    use AuthorizesRequests, DispatchesJobs, ValidatesRequests, ApiResponseTrait;

    public function checkToken($userid, $usertoken)
    {
        $verifiedToken = User::query()
            ->select('id')
            ->where('id', $userid)
            ->where('userToken', $usertoken)
            ->first();

        return $verifiedToken ? $verifiedToken->id : null;
    }

    /**
     * Get formatted user data for API responses
     * 
     * @param \App\Models\User $user The user model instance
     * @return array|null Formatted user data or null if user is invalid
     */
    public function getUserData(User $user): ?array
    {
        if (!$user) {
            return null;
        }
        $businessData = [];
      if($user->hasBusiness == 1){
        $business = Business::query()->where('userId',$user->id)->where('isActive',1)->get();    
        if(count($business) > 0){
            foreach($business as $business){
                $businessData[] = [
                    'businessId' => $business->id,
                    'businessName' => $business->businessName,
                    'businessCategoryId' => $business->businessCategoryId,
                    'businessDetails' => $business->businessDetails,
                    'businessPhone' => $business->businessPhone,
                    'businessAddress' => $business->businessAddress,
                ];
            }
        }
    }
        $profilePicture = $user->profilePicture 
            ? asset('uploads/userImage/' . $user->profilePicture) 
            : asset('uploads/default-user.jpg');
       
        return [
            'id' => $user->id,
            'userName' => $user->userName,
            'email' => $user->email,
            'contactNumber' => $user->contactNumber,
            'profilePicture' => $profilePicture,
            'isActive' => $user->isActive,
            'createdAt' => $user->created_at,
            'updatedAt' => $user->updated_at,
            'hasBusiness' => $user->hasBusiness,
            'business' => $businessData,
        ];
    }
    public function getProductData($product)
    {
        $productImage = $product->productImage 
            ? asset('uploads/productImage/' . $product->productImage) 
            : asset('uploads/default-user.jpg');
       
        return [
            'productId' => $product->id,
            'productName' => $product->productName,
            'price' => $product->price,
            'qty' => $product->qty,
            'detail' => $product->productDetails,
            'productImage' => $productImage,
            'userId' => $product->userId,
            'businessId' => $product->businessId ?? '',
            'productCategoryId' => $product->productCategoryId,
            'isActive' => $product->isActive,
        ];
    }
}