<?php

namespace App\Http\Controllers\api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Mail;
use App\Models\Product;
use App\Models\User;
use App\Models\Business;
use Exception;
use Carbon\Carbon;

class ProductController extends Controller
{
    public function addProduct(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'userId'            => 'required|exists:users,id',
            'userToken'         => 'required',
            'productName'       => 'required|string|max:255',
            'productCategoryId' => 'nullable|string',
            'productDetails'    => 'nullable|string',
            'price'             => 'required|numeric|min:0',
            'qty'               => 'required|integer|min:1',
        ]);
       
        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $input = $request->all();    
        $verifiedUserId = $this->checkToken($input['userId'], $input['userToken']);

        if ($verifiedUserId != $input['userId']) {
            return response()->json([
                'status' => 0,
                'message' => 'You are not authorized please login again'
            ], 401);
        }
        $filename1 = "";
        $upload_dir_path = public_path() . "/uploads/productImage";

        if (isset($input['productImage'])) {
            $file = $input['productImage'];
            $filename1 = rand(0000, 9999) . $file->getClientOriginalName();
            $file->move($upload_dir_path, $filename1);
        }
        $input['productImage'] = $filename1;

        $product = Product::create($input);

        if ($product->id > 0) {
            
            $product = Product::find($product->id);
            $productData=Controller::getProductData($product);

            return response()->json(['status' => 1, 'message' => 'Product Added Successfully', 'result' => $productData]);
        } else {
            return response()->json(['status' => 0, 'message' => 'Product not Added']);
        }
    }
    public function getProductList(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'userId'    => 'required|exists:users,id',
            'userToken' => 'required',
        ]);
        
        if ($validator->fails()) {
            return response()->json([
                'status' => 0,
                'message' => 'Validation failed',
                'errors' => $validator->errors()
            ], 422);
        }

        $input = $request->all();
        $id = $input['userId'];
        $verifiedUserId = $this->checkToken($input['userId'], $input['userToken']);

        if ($verifiedUserId != $id) {
            return response()->json([
                'status' => 0,
                'message' => 'You are not authorized please login again'
            ], 401);
        }

        $products = Product::where('userId', $id)
                          ->where('isActive', 1)
                          ->get();
       
        if ($products->isEmpty()) {
            return response()->json([
                'status' => 0,
                'message' => 'No products found'
            ], 404);
        }

        $productList = [];
        foreach ($products as $product) {
            $productList[] = $this->getProductData($product);
        }

        return response()->json([
            'status' => 1,
            'message' => 'Products found successfully',
            'result' => $productList
        ]);
    }
    public function getProductDetail(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'userId'    => 'required',
            'userToken' => 'required',
            'productId' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $input = $request->all();
        $id = $input['productId'];

        $verifiedUserId = Controller::checkToken($input['userId'], $input['userToken']);

        if ($verifiedUserId == $id) {
            $products = Product::where('id', $id)->where('isActive',1)->get();
            $productList=[];
            if($products)
            {
                foreach($products as $product)
                {
                    $productList[] = Controller::getProductData($product);
                }
                return response()->json(['status' => 1, 'message' => "Product Data Found", 'result' => $productList]);
            }else 
            {
                return response()->json(['status' => 0, 'message' => 'Product Data Not Found']);
            }
                       
        } else {
            return response()->json(['status' => 0, 'message' => 'You are not authorized please login again'], 401);
        }
    }

    public function searchProduct(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'userId'        => 'required|exists:users,id',
            'userToken'     => 'required',
            'searchWord'    => 'required|string|min:2'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 0,
                'message' => 'Validation failed',
                'errors' => $validator->errors()
            ], 422);
        }

        $input = $request->all();
        $verifiedUserId = $this->checkToken($input['userId'], $input['userToken']);

        if ($verifiedUserId != $input['userId']) {
            return response()->json([
                'status' => 0,
                'message' => 'You are not authorized please login again'
            ], 401);
        }

        $searchWord = $input['searchWord'];
        $products = Product::query()
            ->where(function($query) use ($searchWord) {
                $query->where('productName', 'like', "%{$searchWord}%")
                      ->orWhere('productDetails', 'like', "%{$searchWord}%");
            })
            ->where('userId', $input['userId'])
            ->where('isActive', 1)
            ->get();

        if ($products->isEmpty()) {
            return response()->json([
                'status' => 0,
                'message' => 'No products found matching your search'
            ], 404);
        }

        $productData = $products->map(function($product) {
            return $this->getProductData($product);
        });

        return response()->json([
            'status' => 1,
            'message' => 'Products found successfully',
            'result' => $productData
        ]);
    }
    // public function updateProduct(Request $request)
    // {
    //     $validator = Validator::make($request->all(), [
    //         'userId'            => 'required',
    //         'userToken'         => 'required',
    //         'userName'          => 'required',
    //         'email'             => 'required',
    //         'contactNumber'     => 'required',
    //         'hasBusiness'       => 'required|in:0,1',
    //         'businessName'      => 'required_if:hasBusiness,1',
    //         'businessCategoryId'=> 'required_if:hasBusiness,1',
    //         'businessDetails'   => 'nullable',
    //         'businessPhone'     => 'required_if:hasBusiness,1',
    //         'businessAddress'   => 'required_if:hasBusiness,1'
    //     ]);

    //     if ($validator->fails()) {
    //         return response()->json(['status' => 0, 'error' => $validator->errors()], 403);
    //     }

    //     //-------------------Message------------------------------
    //     $msg1 = "Profile Updated Successfully";
    //     $msg2 = "Profile not Updated";
    //     $msg3 = "You are not authorized please login again";
    //     $msg4 = "User Registered with this email";
    //     //-------------------Message------------------------------

    //     $input = $request->all();
    //     $userId = $input['userId'];
    //     $userToken = $input['userToken'];

    //     $verifiedUserId = Controller::checkToken($userId, $userToken);

    //     if ($verifiedUserId == $userId) {
    //         $user = User::find($userId);

    //         if (!$user) {
    //             return response()->json(['status' => 0, 'message' => $msg3], 401);
    //         }

    //         // Check if the provided email or phone already exists for other users
    //         $existingUser = User::where('email', $input['email'])->where('id', '!=', $userId)->first();

    //         if ($existingUser) {
    //             return response()->json(['status' => 0, 'message' => $msg4]);
    //         }

    //         $upload_dir_path = public_path() . "/uploads/userImage";

    //         if (isset($input['profilePicture'])) {
    //             $file = $input['profilePicture'];
    //             $filename1 = rand(0000, 9999) . $file->getClientOriginalName();
    //             $file->move($upload_dir_path, $filename1);
    //         }

    //         $update = [
    //             'userName' => $input['userName']
    //         ];

    //         if (isset($input['email'])) {
    //             $update['email'] = $input['email'];
    //         }

    //         if (isset($input['contactNumber'])) {
    //             $update['contactNumber'] = $input['contactNumber'];
    //         }

    //         if (isset($input['profilePicture'])) {
    //             $update['profilePicture'] = $filename1;
    //         }
           
    //         // Update the user's profile
    //         if ($user->update($update)) {
    //             // Handle business details
    //             if ($input['hasBusiness'] == 1) {
    //                 // Update or create business details
    //                 $business = Business::updateOrCreate(
    //                     ['user_id' => $userId],
    //                     [
    //                         'business_name' => $input['businessName'],
    //                         'business_category' => $input['businessCategoryId'],
    //                         'business_details' => $input['businessDetails'] ?? null,
    //                         'business_phone' => $input['businessPhone'],
    //                         'business_address' => $input['businessAddress'],
    //                     ]
    //                 );
    //             } else {
    //                 // Delete business details if exists
    //                 Business::where('user_id', $userId)->delete();
    //             }

    //             $userData = Controller::getUserData($user);
    //             // Add business data to response if exists
    //             if ($input['hasBusiness'] == 1) {
    //                 $userData['business'] = $user->business;
    //             }

    //             return response()->json(['status' => 1, 'message' => $msg1, 'result' => $userData]);
    //         } else {
    //             return response()->json(['status' => 0, 'message' => $msg2]);
    //         }
    //     } else {
    //         return response()->json(['status' => 0, 'message' => $msg3], 401);
    //     }
    // }

    public function deleteProduct(Request $request)
    {
        $requestData = $request->all();
        $validator = Validator::make($requestData, [
            'userId' => 'required',
            'userToken' => 'required',
            'productId' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json(['status' => 0, 'error' => $validator->errors()], 403);
        }

        /*-------------------Message------------------------------*/
        $msg1 = "Your product deleted successfully";
        $msg2 = "Your product not deleted";
        $msg3 = "You are not authorized please login again";
        /*-------------------Message------------------------------*/

        $userId = (int)$requestData['userId'];
        $userToken = $requestData['userToken'];
        $productId = $requestData['productId'];

        $verifiedUserId = Controller::checkToken($userId, $userToken);

        if ($verifiedUserId == $userId) {
            $deleteProduct = Product::where('id', $productId)->delete();

            if ($deleteProduct) {
                return response(['status' => 1, 'message' => $msg1], 200);
            } else {
                return response(['status' => 0, 'message' => $msg2], 403);
            }
        } else {
            return response(['status' => 0, 'message' => $msg3], 401);
        }
    }

}