<?php

namespace App\Http\Controllers\adminpanel;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\UserBusinessCategory;
use Validator;

class BusinessCategoryController extends Controller
{
    private $pagination = 10;
    public function manage()
    {
        $data = UserBusinessCategory::paginate($this->pagination);
        return view('adminpanel.managebusinesscategory', compact('data'));
    }
    public function add()
    {
        $data = array('type' => 'add');
        return view('adminpanel.addbusinesscategory', compact('data'));
    }
    public function save(Request $request)
    {
        $input = $request->all();
        $validator = Validator::make($input, $this->getRules('Add', $input), $this->messages());
        if ($validator->fails()) {
            $data = array('type' => 'add', 'input' => $input, 'error' => $validator->messages());
            return view('adminpanel.addbusinesscategory', compact('data'));
            exit();
        }
        $user = UserBusinessCategory::create($input);
        if ($user->id > 0) {
            return redirect()->route('adminpanel.businesscategory.manage')->with('success', 'Business Category Created successfully.');
        } else {
            return redirect()->route('adminpanel.businesscategory.add')->withErrors(['Error creating record. Please try again.']);
        }
    }
    public function edit($id)
    {
        $input = UserBusinessCategory::where('id', '=', $id)->get();
        $data = array('type' => 'edit', 'input' => $input);
        return view('adminpanel.addbusinesscategory', compact('data'));
    }
    public function update(Request $request)
    {
        $input = $request->all();
        $id = $input['id'];
        $validator = Validator::make($input, $this->getRules('Edit', $input), $this->messages());
        if ($validator->fails()) {
            $data = array('type' => 'Edit', 'input' => $input, 'error' => $validator->messages());
            return view('adminpanel.addbusinesscategory', compact('data'));
            exit();
        }
        $update = array();
        $update["categoryName"] = $input['categoryName'];
        $user = UserBusinessCategory::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.businesscategory.manage')->with('success', 'Business Category Updated successfully.');
    }
    public function delete($id)
    {
        $user = UserBusinessCategory::where('id', '=', $id)->delete();
        return redirect()->route('adminpanel.businesscategory.manage')->with('success', 'Business Category Deleted successfully.');
    }
    public function deactive($id)
    {
        $update = array();
        $update["isActive"] = 0;
        $user = UserBusinessCategory::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.businesscategory.manage')->with('success', 'Business User deactive successfully.');
    }
    public function active($id)
    {
        $update = array();
        $update["isActive"] = 1;
        $user = UserBusinessCategory::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.businesscategory.manage')->with('success', 'Business User active successfully.');
    }
    public function search(Request $request)
    {
        $input = $request->all();
        $qry = UserBusinessCategory::query();
        if (trim($input["search"]) != "") {
            $search = $input["search"];
            $qry->where([
                ["categoryName", "like", "%{$search}%"],
            ]);

        }
        $data = $qry->paginate($this->pagination);
        $data->appends($input);
        return view('adminpanel.managebusinesscategory', compact('data'));
    }
    private function getRules($type, $input)
    {
        $return = array();
        $return['categoryName'] = 'required';
        return $return;
    }
    private function messages()
    {
        return [
            'categoryName.required'  => $this->getRequiredMessage('Category Name'),
        ];
    }
    private function getRequiredMessage($string)
    {
        return 'The ' . $string . ' field is required.';
    }

}