<?php

namespace App\Http\Controllers\adminpanel;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\ProductCategory;
use Validator;

class ProductCategoryController extends Controller
{
    private $pagination = 10;

    public function manage()
    {
        $data = ProductCategory::paginate($this->pagination);
        return view('adminpanel.manageproductcategory', compact('data'));
    }

    public function add()
    {
        $data = array('type' => 'add');
        return view('adminpanel.addproductcategory', compact('data'));
    }

    public function save(Request $request)
    {
        $input = $request->all();
        $validator = Validator::make($input, $this->getRules('Add', $input), $this->messages());
        if ($validator->fails()) {
            $data = array('type' => 'add', 'input' => $input, 'error' => $validator->messages());
            return view('adminpanel.addproductcategory', compact('data'));
            exit();
        }
        $category = ProductCategory::create($input);
        if ($category->id > 0) {
            return redirect()->route('adminpanel.productcategory.manage')->with('success', 'Product Category Created successfully.');
        } else {
            return redirect()->route('adminpanel.productcategory.add')->withErrors(['Error creating record. Please try again.']);
        }
    }

    public function edit($id)
    {
        $input = ProductCategory::where('id', '=', $id)->get();
        $data = array('type' => 'edit', 'input' => $input);
        return view('adminpanel.addproductcategory', compact('data'));
    }

    public function update(Request $request)
    {
        $input = $request->all();
        $id = $input['id'];
        $validator = Validator::make($input, $this->getRules('Edit', $input), $this->messages());
        if ($validator->fails()) {
            $data = array('type' => 'Edit', 'input' => $input, 'error' => $validator->messages());
            return view('adminpanel.addproductcategory', compact('data'));
            exit();
        }
        $update = array();
        $update["categoryName"] = $input['categoryName'];
        $category = ProductCategory::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.productcategory.manage')->with('success', 'Product Category Updated successfully.');
    }

    public function delete($id)
    {
        $category = ProductCategory::where('id', '=', $id)->delete();
        return redirect()->route('adminpanel.productcategory.manage')->with('success', 'Product Category Deleted successfully.');
    }

    public function deactive($id)
    {
        $update = array();
        $update["isActive"] = 0;
        $category = ProductCategory::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.productcategory.manage')->with('success', 'Product Category deactivated successfully.');
    }

    public function active($id)
    {
        $update = array();
        $update["isActive"] = 1;
        $category = ProductCategory::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.productcategory.manage')->with('success', 'Product Category activated successfully.');
    }

    public function search(Request $request)
    {
        $input = $request->all();
        $qry = ProductCategory::query();
        if (trim($input["search"]) != "") {
            $search = $input["search"];
            $qry->where([
                ["categoryName", "like", "%{$search}%"],
            ]);
        }
        $data = $qry->paginate($this->pagination);
        $data->appends($input);
        return view('adminpanel.manageproductcategory', compact('data'));
    }

    private function getRules($type, $input)
    {
        $return = array();
        $return['categoryName'] = 'required';
        return $return;
    }

    private function messages()
    {
        return [
            'categoryName.required'  => $this->getRequiredMessage('Category Name'),
        ];
    }

    private function getRequiredMessage($string)
    {
        return 'The ' . $string . ' field is required.';
    }
}