<?php

namespace App\Http\Controllers\adminpanel;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Mail;
use App\Models\User;
use App\Models\Country;
use App\Models\Occupation;
use App\Models\Qualification;
use App\Models\Religion;
use Validator;
use Illuminate\Pagination\Paginator;
use Carbon\Carbon;
use Illuminate\Validation\Rule;
use Log;

class UserController extends Controller
{
    private $pagination = 10;
    public function manage()
    {
        $data = User::where('isActive', '!=', 0)->paginate($this->pagination);
        return view('adminpanel.manageuser', compact('data'));
    }
    public function add()
    {
        $data = array('type' => 'add');

        return view('adminpanel.adduser', compact('data'));
    }
    public function save(Request $request)
    {
        $input = $request->all();


        $validator = Validator::make($input, $this->getRules('Add', $input), $this->messages());
        if ($validator->fails()) {


            $data = array('type' => 'add', 'input' => $input, 'error' => $validator->messages());
            return view('adminpanel.adduser', compact('data'));
            exit();
        }
        $input['password'] = bcrypt($input['password']);

        $upload_dir_path = public_path() . "/uploads/userImage";
        if (isset($input['profilePicture'])) {
            $file = $input['profilePicture'];
            $filename1 = rand(0000, 9999) . $file->getClientOriginalName();
            $file->move($upload_dir_path, $filename1);
            $input['profilePicture'] = $filename1;
        }

        $user = User::create($input);
        if ($user->id > 0) {

            return redirect()->route('adminpanel.user.manage')->with('success', 'User Created successfully.');
        } else {
            return redirect()->route('adminpanel.user.add')->withErrors(['Error creating record. Please try again.']);
        }
    }
    public function edit($id)
    {
        $input = User::where('id', '=', $id)->get();
        $data = array('type' => 'edit', 'input' => $input);

        return view('adminpanel.adduser', compact('data'));
    }
    public function update(Request $request)
    {
        $input = $request->all();
        $id = $input['id'];

        $validator = Validator::make($input, $this->getRules('Edit', $input), $this->messages());
        if ($validator->fails()) {

            $data = array('type' => 'Edit', 'input' => $input, 'error' => $validator->messages());
            return view('adminpanel.adduser', compact('data'));
            exit();
        }
        $update = array();
        $update["userName"] = $input['userName'];

        $update["contactNumber"] = $input['contactNumber'];
        $update["email"] = $input['email'];
        if (isset($input['password'])) {
            $update["password"] = bcrypt($input['password']);
        }

        if (isset($input['profilePicture'])) {
            $file = $input['profilePicture'];
            $filename1 = rand(0000, 9999) . $file->getClientOriginalName();
            $file->move($upload_dir_path, $filename1);
            $update['profilePicture'] = $filename1;
        }

        $user = User::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.user.manage')->with('success', 'User Updated successfully.');
    }
    public function delete($id)
    {

        $user = User::where('id', '=', $id)->delete();
        return redirect()->route('adminpanel.user.manage')->with('success', 'User Deleted successfully.');
    }
    public function deactive($id)
    {
        $update = array();
        $update["isActive"] = 0;
        $user = User::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.user.manage')->with('success', 'User Block successfully.');
    }
    public function active($id)
    {
        $update = array();
        $update["isActive"] = 1;
        $user = User::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.user.manage')->with('success', 'User Un Block successfully.');
    }
    public function search(Request $request)
    {
        $input = $request->all();
        $qry = User::query();
        if (trim($input["search"]) != "") {
            $search = $input["search"];
            $qry->where([
                ["userName", "like", "%{$search}%"],
            ]);
            $qry->orwhere([
                ["email", "like", "%{$search}%"],
            ]);
        }
        $data = $qry->paginate($this->pagination);
        $data->appends($input);
        return view('adminpanel.manageuser', compact('data'));
    }
    private function getRules($type, $input)
    {
        $return = array();

        $return['userName'] = 'required';

        $return['contactNumber'] =  'required|max:20';

        $return['email'] = [
            'required',
            'email',
            'max:100',
            Rule::unique('users', 'email')->where(function ($query) use ($input) {
                $query->where('isActive', '!=', 0)->where('id', '!=', $input['id']);
            })
        ];
        // Image validation for both Add and Edit
        if ($type === 'Add') {
            // Required for new users
            $return['profilePicture'] = [
                'nullable',
                'image',
                'mimes:jpeg,png,jpg',
                'max:2048' // 2MB max size
            ];
            $return['password'] = 'required|min:6|max:20';
        } else {
            // Optional for editing users
            if (isset($input['profilePicture'])) {
                $return['profile_picture'] = [
                    'nullable',
                    'image',
                    'mimes:jpeg,png,jpg',
                    'max:2048'
                ];
            }
        }
        return $return;
    }
    private function messages()
    {
        return [

            'userName.required'  => $this->getRequiredMessage('username'),
            'username.max'  => $this->getGreaterMessage('username', 30),

            'contactNumber.required'  => $this->getRequiredMessage('contact number'),
            'contactNumber.max'  => $this->getGreaterMessage('contact number', 20),
            'email.required'  => $this->getRequiredMessage('email'),
            'email.max'  => $this->getGreaterMessage('email', 100),
            'email.unique' => 'The email address has already been taken.',
            // Add image validation messages
            // Update image validation messages
            'profilePicture.image' => 'The profile picture must be an image.',
            'profilePicture.mimes' => 'The profile picture must be a file of type: jpeg, png, jpg.',
            'profilePicture.max' => 'The profile picture must not be larger than 2MB.'
        ];
    }
    private function getRequiredMessage($string)
    {
        return 'The ' . $string . ' field is required.';
    }
    private function getGreaterMessage($string, $maxchar)
    {
        return 'The ' . $string . ' may not be greater than ' . $maxchar . ' characters.';
    }
    public function validateForm(Request $request)
    {
        $input = $request->all();
        $type = isset($input['id']) ? 'Edit' : 'Add';

        try {
            $validator = Validator::make($input, $this->getRules($type, $input), $this->messages());

            if ($validator->fails()) {
                return response()->json([
                    'success' => false,
                    'errors' => $validator->errors()
                ], 422);
            }

            return response()->json([
                'success' => true
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'success' => false,
                'errors' => ['general' => [$e->getMessage()]]
            ], 500);
        }
    }
}
