<?php

namespace App\Http\Controllers\api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Mail;
use App\Models\User;
use App\Models\Product;
use App\Models\UserBusiness;
use Exception;
use Carbon\Carbon;

class UserController extends Controller
{
    public function register(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'userName'              => 'required|min:3',
            'email'                 => 'required',
            'contactNumber'         => 'required|min:10',
            'password'              => 'required|min:6',
            ]);

        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $input = $request->all();

        $input['password'] = bcrypt($input['password']);
        $input['userToken'] = md5(rand(1, 999999));
        $filename1 = "";
        $upload_dir_path = public_path() . "/uploads/userImage";

        if (isset($input['profilePicture'])) {
            $file = $input['profilePicture'];
            $filename1 = rand(0000, 9999) . $file->getClientOriginalName();
            $file->move($upload_dir_path, $filename1);
        }
        $input['profilePicture'] = $filename1;

        if ($this->isUserExists($input['email'])) {
            return response()->json(['status' => 0, 'message' => 'User already registered with this email']);
        }

        // if ($this->isUserPhoneExists($contact_number, $country_id)) {
        //     return response()->json(['status' => 0, 'message' => 'User already registered with this phone number & country code']);
        // }

        $user = User::create($input);

        if ($user->id > 0) {
            
            $users = User::find($user->id);
            $userData=Controller::getUserData($users);

            return response()->json(['status' => 1, 'message' => 'User Registered Successfully', 'result' => $userData]);
        } else {
            return response()->json(['status' => 0, 'message' => 'User not registered']);
        }
    }

    private function isUserExists($email)
    {
        return User::where('email', $email)->where('isActive', '=', 1)->exists();
    }

    private function isUserPhoneExists($contact_number, $country_id)
    {
        return User::where('contact_number', $contact_number)->where('country_id', $country_id)->where('is_active', '!=', 0)->exists();
    }

  
    public function login(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email'                 => 'required',
            'password'              => 'required|min:6',
            ]);

        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 403);
        }
        $email=$request->email;
        $password = $request->password;

        if (Auth::guard('web')->attempt(['email' => $email, 'password' => $password], false)) {
              
            $user = Auth::user();
            $userData = Controller::getUserData($user);

            if ($userData['isActive'] == 0) {
                return response()->json(['status' => 0, 'message' => "Your Account Is Deactivate"]);
            } 
                      
            $tkn = md5(rand(1, 999999));
            $user->update(['userToken' => $tkn]);
            $userData['userId'] = $user['id'];
            $userData['userToken'] = $tkn;

            return response()->json(['status' => 1, 'message' => "Login Successfully", 'result' => $userData]);
        }else
        {
            return response()->json(['status' => 0, 'message' => "Email or Password is incorrect. Please try again!"]);
        }
       
    }

    public function forgotPassword(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $email = $request->input('email');

        $user = User::where('email', $email)->first();

        if ($user) {
            $newPassword = rand(100000, 999999);
            $user->update(['password' => bcrypt($newPassword)]);

            $mail = $email;
            $data['username'] = $user->username;
            $data['new_password'] = $newPassword;

            Mail::send('adminpanel.forgotpassword', $data, function ($message) use ($mail) {
                $message->from('kitest@keshavinfotechdemo2.com', 'BridgeApp');
                $message->to($mail)->subject('Forgot Password');
            });

            $success = [
                'user_id' => $user->id,
                'email' => $user->email,
            ];

            return response()->json(['status' => 1, 'message' => "Mail sent Successfully", 'result' => $success]);
        } else {
            return response()->json(['status' => 0, 'message' => "Mail not Found"]);
        }
    }

    public function changePassword(Request $request)
    {
        $requestData = $request->all();

        $validator = Validator::make($requestData, [
            'userId'        => 'required',
            'userToken'     => 'required',
            'oldPassword'   => 'required|min:6',
            'newPassword'   => 'required|min:6',
        ]);

        if ($validator->fails()) {
            return response()->json(['status' => 0, 'error' => $validator->errors()], 403);
        }

        /*-------------------Message------------------------------*/
        $msg1 = "Password changed successfully";
        $msg2 = "Password not changed";
        $msg3 = "Old Password does not match";
        $msg4 = "You are not authorized please login again";
        /*-------------------Message------------------------------*/

        $userId = (int)$requestData['userId'];
        $userToken = $requestData['userToken'];
        $oldPassword = $requestData['oldPassword'];
        $userData['password'] = bcrypt($requestData['newPassword']);

        $verifiedUserId = Controller::checkToken($userId, $userToken);

        if ($verifiedUserId == $userId) {
            $user = User::where('id', $userId)->get();
            if (Hash::check($oldPassword, $user[0]->password)) {
                $changePassword = User::where('id', $userId)->update($userData);

                if ($changePassword) {
                    return response()->json(['status' => 1, 'message' => $msg1]);
                } else {
                    return response()->json(['status' => 0, 'message' => $msg2], 403);
                }
            } else {
                return response()->json(['status' => 0, 'message' => $msg3], 403);
            }
        } else {
            return response()->json(['status' => 0, 'message' => $msg4], 401);
        }
    }

    public function getUserProfile(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'userId'        => 'required|exists:users,id',
            'userToken'     => 'required',
            'otherUserId'   => 'nullable|exists:users,id'
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 403);
        }

        $input = $request->all();
        $id = $input['userId'];
        $otherUserId = $input['otherUserId'] ?? $id;
        $verifiedUserId = Controller::checkToken($input['userId'], $input['userToken']);

        if ($verifiedUserId != $id) {
            return response()->json([
                'status' => 0, 
                'message' => 'You are not authorized please login again'
            ], 401);
        }

        $user = User::where('id', $otherUserId)
                    ->where('isActive', 1)
                    ->first();

        if (!$user) {
            return response()->json([
                'status' => 0, 
                'message' => 'User not found' 
            ], 404);
        }

        $userData = Controller::getUserData($user);
        
        // Get products if they exist
        $products = Product::where('userId', $otherUserId)
                          ->where('isActive', 1)
                          ->get();
        
        if ($products->isNotEmpty()) {
            $productList = [];
            foreach ($products as $product) {
                $productList[] = [
                    'productId' => $product->id,
                    'productName' => $product->productName,
                    'price' => $product->price,
                    'qty' => $product->qty,
                    'detail' => $product->productDetails,
                    'productImage' => $product->productImage ? asset('uploads/productImage/' . $product->productImage) : asset('uploads/default-user.jpg'),
                    'isActive' => $product->isActive
                ];
            }
            $userData['productList'] = $productList;
        }

        return response()->json([
            'status' => 1, 
            'message' => 'User profile retrieved successfully', 
            'result' => $userData
        ]);
    }

    public function updateProfile(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'userId'            => 'required',
            'userToken'         => 'required',
            'userName'          => 'required',
            'email'             => 'required',
            'contactNumber'     => 'required',
            'hasBusiness'       => 'required|in:0,1',
            'businessName'      => 'required_if:hasBusiness,1',
            'businessCategoryId'=> 'required_if:hasBusiness,1',
            'businessDetails'   => 'nullable',
            'businessPhone'     => 'required_if:hasBusiness,1',
            'businessAddress'   => 'required_if:hasBusiness,1'
        ]);

        if ($validator->fails()) {
            return response()->json(['status' => 0, 'error' => $validator->errors()], 403);
        }

        //-------------------Message------------------------------
        $msg1 = "Profile Updated Successfully";
        $msg2 = "Profile not Updated";
        $msg3 = "You are not authorized please login again";
        $msg4 = "User Registered with this email";
        //-------------------Message------------------------------

        $input = $request->all();
        $userId = $input['userId'];
        $userToken = $input['userToken'];

        $verifiedUserId = Controller::checkToken($userId, $userToken);

        if ($verifiedUserId == $userId) {
            $user = User::find($userId);

            if (!$user) {
                return response()->json(['status' => 0, 'message' => $msg3], 401);
            }

            // Check if the provided email or phone already exists for other users
            $existingUser = User::where('email', $input['email'])->where('id', '!=', $userId)->first();

            if ($existingUser) {
                return response()->json(['status' => 0, 'message' => $msg4]);
            }

            $upload_dir_path = public_path() . "/uploads/userImage";

            if (isset($input['profilePicture'])) {
                $file = $input['profilePicture'];
                $filename1 = rand(0000, 9999) . $file->getClientOriginalName();
                $file->move($upload_dir_path, $filename1);
            }

            $update = [
                'userName' => $input['userName']
            ];

            if (isset($input['email'])) {
                $update['email'] = $input['email'];
            }

            if (isset($input['contactNumber'])) {
                $update['contactNumber'] = $input['contactNumber'];
            }

            if (isset($input['profilePicture'])) {
                $update['profilePicture'] = $filename1;
            }
           
            // Update the user's profile
            if ($user->update($update)) {
                // Handle business details
                if ($input['hasBusiness'] == 1) {
                    // Update or create business details
                    $business = Business::updateOrCreate(
                        ['userId' => $userId],
                        [
                            'businessName' => $input['businessName'],
                            'businessCategoryId' => $input['businessCategoryId'],
                            'businessDetails' => $input['businessDetails'] ?? null,
                            'businessPhone' => $input['businessPhone'],
                            'businessAddress' => $input['businessAddress'],
                        ]
                    );
                } else {
                    // Delete business details if exists
                    UserBusiness::where('userId', $userId)->delete();
                }

                $userData = Controller::getUserData($user);
                // Add business data to response if exists
                if ($input['hasBusiness'] == 1) {
                    $userData['business'] = $user->business;
                }

                return response()->json(['status' => 1, 'message' => $msg1, 'result' => $userData]);
            } else {
                return response()->json(['status' => 0, 'message' => $msg2]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => $msg3], 401);
        }
    }

    public function deleteAccount(Request $request)
    {
        $requestData = $request->all();
        $validator = Validator::make($requestData, [
            'userId' => 'required',
            'userToken' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json(['status' => 0, 'error' => $validator->errors()], 403);
        }

        /*-------------------Message------------------------------*/
        $msg1 = "Your account deleted successfully";
        $msg2 = "Your account not deleted";
        $msg3 = "You are not authorized please login again";
        /*-------------------Message------------------------------*/

        $userId = (int)$requestData['userId'];
        $userToken = $requestData['userToken'];

        $verifiedUserId = Controller::checkToken($userId, $userToken);

        if ($verifiedUserId == $userId) {
            $deleteAccount = User::where('id', $userId)->delete();

            if ($deleteAccount) {
                return response(['status' => 1, 'message' => $msg1], 200);
            } else {
                return response(['status' => 0, 'message' => $msg2], 403);
            }
        } else {
            return response(['status' => 0, 'message' => $msg3], 401);
        }
    }

    public function logout(Request $req)
    {
        $validator = Validator::make($req->all(), [
            'userId'    => 'required',
            'userToken' => 'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['status' => 1, 'message' => $validator->errors()]);
        }
        $data = User::where('id', $req->userId)->where('userToken', $req->userToken)->get();
        if ($data->count()) {
            $update['userToken'] = '';
            User::where('id', $req->userId)->update($update);
            return response()->json(['status' => 1, 'message' => 'Logout successfully']);
            exit;
        } else {
            return response()->json(['status' => 0, 'message' => 'Logout failed']);
            exit;
        }
    }

}